#ifndef FIELDI_H
#define FIELDI_H
//quadratic extension of a field that has no square root of -1

#include "pbc.h"

struct fi_field_data_s {
    field_ptr field;
};
typedef struct fi_field_data_s *fi_field_data_ptr;
typedef struct fi_field_data_s fi_field_data_t[1];

struct fi_data_s {
    //elements have the form x + yi
    element_t x;
    element_t y;
};
typedef struct fi_data_s *fi_data_ptr;
typedef struct fi_data_s fi_data_t[1];

void field_init_fi(field_ptr f, field_ptr fbase);

//naturally map an element from a field K to K[i]
void element_field_to_fieldi(element_ptr r, element_ptr a);

static inline element_ptr fi_re(element_ptr a)
{
    return ((fi_data_ptr) a->data)->x;
}

static inline element_ptr fi_im(element_ptr a)
{
    return ((fi_data_ptr) a->data)->y;
}

#endif //FIELDI_H
